package vismon.data;

import org.msgpack.annotation.MessagePackMessage;
import vismon.util.Constants;
import vismon.util.Util;

import java.util.*;

//@MessagePackMessage
public class DataItem {

    private final int CDF_BIN_NUM = 100;//Session.getSimulationNum();

    protected MngAction mngAction;
    protected IndicatorValues[] indValues;  // length = simulation number
    protected Map<Indicator, double[]> mirroredIndNNaNValues;

    protected Map<Indicator, double[]> minMaxs;
    protected IndicatorValues averages;
    protected IndicatorValues medians;
    protected Map<Indicator, double[]> quartiles;
    protected Map<Indicator, Integer> nanNumbers;

    protected boolean filtered;

    protected boolean pdfComputed;
    protected Map<Indicator, Double> pdfWholeArea;
    protected Map<Indicator, List<Double>> pdfAreas;
    protected Map<Indicator, List<Double>> pdfValues;
    protected Map<Indicator, Double> pdfMax;
    protected Map<Indicator, Map<Integer, double[]>> errorBar;

    //protected Map<Indicator, Boolean>         fulfillsIndProbValues;

    public DataItem() {
        mngAction = new MngAction();

        indValues = new IndicatorValues[Session.getSimulationNum()];
        for (int i = 0; i < indValues.length; ++i)
            indValues[i] = new IndicatorValues();

        mirroredIndNNaNValues = new HashMap<Indicator, double[]>();

        minMaxs = new HashMap<Indicator, double[]>();
        averages = new IndicatorValues();
        medians = new IndicatorValues();
        quartiles = new HashMap<Indicator, double[]>();
        nanNumbers = new HashMap<Indicator, Integer>();

        filtered = false;

        pdfComputed = false;
        pdfWholeArea = new HashMap<Indicator, Double>();
        pdfAreas = new HashMap<Indicator, List<Double>>();
        pdfValues = new HashMap<Indicator, List<Double>>();
        pdfMax = new HashMap<Indicator, Double>();
        errorBar = new HashMap<Indicator, Map<Integer, double[]>>();

        /*fulfillsIndProbValues = new HashMap<Indicator, Boolean>();
        for (Indicator dim : Session.getIndicators())
            fulfillsIndProbValues.put(dim, true);*/
    }

    public DataItem(MngAction action) {
        this();
        mngAction = action;
    }

    public MngAction getMngAction() {
        return mngAction;
    }

    public boolean isFiltered() {
        for (Indicator indicator : Session.getIndicators()) {
            double value = getAverageValue(indicator);
            if (indicator.isMaxBest()) {
                if (value < indicator.getConstraintValue())
                    return true;
            } else {
                if (value > indicator.getConstraintValue())
                    return true;
            }

        }
        return false;
        //return filtered;
    }

    public boolean isFullFillsProbability(Indicator indicator, double percentage) {
        if (Session.getSimulationNum() == nanNumbers.get(indicator))
            return true;

        double fullfillingSimNum = Arrays.binarySearch(mirroredIndNNaNValues.get(indicator),
                indicator.getConstraintValue());
        if (fullfillingSimNum < 0) {
            fullfillingSimNum = -fullfillingSimNum - 1;
        }
        if (indicator.isMaxBest())
            fullfillingSimNum = (Session.getSimulationNum() - nanNumbers.get(indicator)) - fullfillingSimNum;

        double prob = fullfillingSimNum / (Session.getSimulationNum() - nanNumbers.get(indicator));

        if (prob >= percentage)
            return true;

        return false;
    }

    public void setMirroredIndValues() {
        for (Indicator ind : Session.getIndicators()) {
            double[] values = new double[Session.getSimulationNum() - nanNumbers.get(ind)];
            int index = 0;
            for (int i = 0; i < Session.getSimulationNum(); ++i) {
                double value = indValues[i].getValue(ind);
                if (!Util.isNaN(value))
                    values[index++] = value;
            }

            Arrays.sort(values);
            mirroredIndNNaNValues.put(ind, values);
        }
    }

    public boolean hasMngAction(MngAction action) {
        return mngAction.equals(action);
    }

    public void addValue(int simNum, Indicator ind, double value) {
        indValues[simNum].addValue(ind, value);

        // FINDING MIN AND MAX VALUES OF ALL SIMULATIONS FOR "DIM"
        if (ind.getMin() > value)
            ind.setMin(value);
        if (ind.getMax() < value)
            ind.setMax(value);
    }

    public double getValue(int simNum, Indicator ind) {
        return indValues[simNum].getValue(ind);
    }

    public double[] getValues(Indicator ind) {
        return mirroredIndNNaNValues.get(ind);
    }

    public boolean hasValue(int simNum, Indicator ind, double v) {
        return (Util.equals(v, mirroredIndNNaNValues.get(ind)[simNum]));
    }

    public double getAverageValue(Indicator ind) {
        return averages.getValue(ind);
    }

    public double getNormalizedAverageValue(Indicator ind) {
        return (averages.getValue(ind) - ind.getMin()) / (ind.getMax() - ind.getMin());
    }

    public boolean hasAverageValue(Indicator ind, double v) {
        return averages.hasValue(ind, v);
    }

    public double getMedianValue(Indicator ind) {
        return medians.getValue(ind);
    }

    public double getMinValue(Indicator ind) {
        return minMaxs.get(ind)[0];
    }

    public double getMaxValue(Indicator ind) {
        return minMaxs.get(ind)[1];
    }

    public double[] getQuartiles(Indicator ind) {
        return quartiles.get(ind);
    }

    ////////////////////////////////////////////////////////////////////////////

    public void doComputations() {
        countNaNNumbers();
        setMirroredIndValues();
        computeMinMaxs();
        computeAverages();
        computeMedians();
        computeQuartiles();
        computePDF();
    }

    public void computeMinMaxs() {
        for (Indicator ind : Session.getIndicators())
            minMaxs.put(ind, computeMinMax(ind));
    }

    public void computeAverages() {
        for (Indicator ind : Session.getIndicators())
            averages.addValue(ind, computeAverage(ind));
    }

    public void computeMedians() {
        for (Indicator ind : Session.getIndicators())
            medians.addValue(ind, computeMedian(ind));
    }
    public void computeQuartiles() {
        for (Indicator ind : Session.getIndicators())
            quartiles.put(ind, computeQuartile(ind));
    }

    public void countNaNNumbers() {
        for (Indicator ind : Session.getIndicators())
            nanNumbers.put(ind, countNaNNumber(ind));
    }

    public double[] computeMinMax(Indicator ind) {
        double min = Double.MAX_VALUE;
        double max = Double.MIN_VALUE;
        for (double v : mirroredIndNNaNValues.get(ind)) {
            if (v < min)
                min = v;
            if (v > max)
                max = v;
        }
        return new double[]{min, max};
    }

    public double computeAverage(Indicator ind) {
        double[] values = mirroredIndNNaNValues.get(ind);

        int num = values.length;
        if (num == 1)
            return values[0];

        double sum = 0.0;
        for (double v : values)
            sum += v;
        return sum / num;
    }

    public double computeMedian(Indicator ind) {
        double[] values = mirroredIndNNaNValues.get(ind);
        int num = values.length;
        if (num == 0)
            return Constants.NA;

        return values[(int) (0.5 * num)];
    }

    public double[] computeQuartile(Indicator ind) {
        double values[] = mirroredIndNNaNValues.get(ind);
        int num = values.length;
        if (num == 0)
            return new double[]{Constants.NA, Constants.NA};

        return new double[]{values[(int) (0.25 * num)], values[(int) (0.75 * num)]};
    }

    private int countNaNNumber(Indicator indicator) {
        int nanNum = 0;
        for (int i = 0; i < Session.getSimulationNum(); ++i)
            if (Util.isNaN(getValue(i, indicator)))
                ++nanNum;
        return nanNum;
    }

    public double[] getErrorBar(Indicator ind, int percent) {
        Map<Integer, double[]> error = errorBar.get(ind);
        if (error != null && error.get(percent) != null)
            return error.get(percent);

        double[] ci = computeErrorBar(ind, percent);
        if (error == null)
            error = new HashMap<Integer, double[]>();
        error.put(percent, ci);
        errorBar.put(ind, error);

        return ci;
    }

    public double[] computeErrorBar(Indicator ind, int percent) {
        double[] errorBarBounds = new double[2];

        double min = minMaxs.get(ind)[0];
        double max = minMaxs.get(ind)[1];

        double a0 = (max - min) / (CDF_BIN_NUM - 1);

        double pnPercent = 0.5 - percent * 0.005;

        List<Double> fAreas = pdfAreas.get(ind);
        double fWholeArea = pdfWholeArea.get(ind);
        int fAreasNum = fAreas.size();

        double prevArea = 0;
        for (int i = 0; i < fAreasNum; ++i) {
            prevArea += fAreas.get(i);
            if ((prevArea / fWholeArea) >= pnPercent) {
                errorBarBounds[0] = min + (i - 1) * a0;
                break;
            }
        }

        double nextArea = 0;
        for (int i = fAreasNum - 1; i >= 0; --i) {
            nextArea += fAreas.get(i);
            if ((nextArea / fWholeArea) >= pnPercent) {
                errorBarBounds[1] = min + (i + 1) * a0;
                break;
            }
        }
        return errorBarBounds;
    }


    public void computePDF() {
        double beginningTime = System.currentTimeMillis();
        if (pdfComputed)
            return;

        for (Indicator ind : Session.getIndicators()) {
            double[] values = getValues(ind);
            List<Double> fValues = new ArrayList<Double>();

            double minValue = Double.MAX_VALUE;
            double maxValue = Double.MIN_VALUE;

            for (int i = 0; i < values.length; ++i) {
                if (values[i] < minValue)
                    minValue = values[i];
                if (values[i] > maxValue)
                    maxValue = values[i];
            }

            double range = maxValue - minValue;
            double a0 = range / (CDF_BIN_NUM - 1);
            double fMax = Double.MIN_VALUE;

            double windowSize = 0.1 * range;
            double sd = windowSize;

            /*Map<Set, Double> preCalculatedValues = new HashMap<Set, Double>();
            for (int n = 0; n < CDF_BIN_NUM; ++n) {
                double y_n = minValue + n * a0;

                for (int p = 0; p < values.length; ++p) {
                    if (Util.isNaN(values[p]))
                        continue;

                    double v;
                    if ((values[p] - y_n) > windowSize || (y_n - values[p]) > windowSize)
                        v = 0.0;
                    else
                        v = (windowSize - Math.abs(values[p] - y_n)) / (windowSize * windowSize);
                    Set<Double> points = new HashSet<Double>();
                    points.add(y_n);
                    points.add(values[p]);
                    preCalculatedValues.put(points, v);
                }
            }

            for (int n = 0; n < CDF_BIN_NUM; ++n) {
                double y_n = minValue + n * a0;
                double func_y_n = 0;

                for (int p = 0; p < values.length; ++p) {
                    if (Util.isNaN(values[p]))
                        continue;

                    Set<Double> points = new HashSet<Double>();
                    points.add(y_n);
                    points.add(values[p]);

                    if (!preCalculatedValues.containsKey(points))
                        System.out.println("not containing <" + y_n + " - " + values[p] + ">");

                    func_y_n += preCalculatedValues.get(points);
                }
                func_y_n /= (values.length);
                fValues.add(func_y_n);

                if (fMax < func_y_n)
                    fMax = func_y_n;
            }*/

            int stepNum = 10;
            double[] tentValuesArray = new double[stepNum];

            for (int i = 0; i < stepNum; ++i) {
                double dif = i * windowSize / stepNum;
                double tentV = (windowSize - dif) / (windowSize * windowSize);
                tentValuesArray[i] = tentV;
            }

            for (int n = 0; n < CDF_BIN_NUM; ++n) {
                double y_n = minValue + n * a0;
                double func_y_n = 0;

                for (int p = 0; p < values.length; ++p) {
                    if (Util.isNaN(values[p]) ||
                        values[p] < (y_n - windowSize) || values[p] > (y_n + windowSize))
                        continue;

                    //func_y_n += Util.getGaussian(values[p], y_n, sd);
                    //func_y_n += Util.getTent(values[p], y_n, windowSize);
                    double dif = values[p] - y_n;
                    if (dif < 0)
                        dif = -dif;
                    int i = (int)((dif * stepNum) / windowSize);
                    func_y_n += tentValuesArray[i];
                }
                func_y_n /= (values.length);
                fValues.add(func_y_n);

                if (fMax < func_y_n)
                    fMax = func_y_n;
            }

            pdfValues.put(ind, fValues);
            pdfMax.put(ind, fMax);

            //CALCULATING AREAS OF FUNCTION TRAPEZOIDS
            List<Double> fAreas = new ArrayList<Double>();
            double wholeArea = 0;

            for (int i = 0; i < fValues.size() - 1; ++i) {
                double area = (fValues.get(i) + fValues.get(i + 1)) * a0 * 0.5;
                fAreas.add(area);
                wholeArea += area;
            }
            pdfAreas.put(ind, fAreas);
            pdfWholeArea.put(ind, wholeArea);
        }

        pdfComputed = true;
        //System.out.println("takes = " + (System.currentTimeMillis() - beginningTime));
    }

    public Map<Indicator, Double> getPdfMax() {
        return pdfMax;
    }

    public Map<Indicator, List<Double>> getPdfValues() {
        return pdfValues;
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof DataItem) {
            DataItem oDataItem = (DataItem) o;
            return mngAction.equals(oDataItem.getMngAction());
        }
        return false;
    }

    @Override
    public String toString() {
        return mngAction.toString();
    }

}
