package vismon.gui;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

public class About {

    private JFrame frame;

    private JPanel mainPanel;

    protected void run() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                setUp();
                start();
            }
        });
    }

    protected void setUp() {
        initComponents();
    }

    protected void start() {
        addPanel();

        frame.setVisible(true);
    }

    protected void initComponents() {
        int width = 350;
        int height = 300;

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int xLocation = (int)((screenSize.getWidth() - width) * 0.5);
        int yLocation = (int)((screenSize.getHeight() - height) * 0.5);

        this.frame = new JFrame("Preferences");
        this.frame.setSize(new Dimension(width, height));
        this.frame.setLocation(xLocation, yLocation);
        this.frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.frame.getContentPane().setLayout(new TableLayout(new double[][]{{0, -1, 0}, {0, -1, 0}}));
    }

    protected void addPanel() {
        mainPanel = new JPanel();
        mainPanel.setBackground(Color.WHITE);

        double[][] mainLayout = new double[][]{{TableLayoutConstants.FILL},
                {0.56, 0.3, 0.04, 0.1}};
        mainPanel.setLayout(new TableLayout(mainLayout));

        addLogo();
        addText();
        addOKButton();

        Container frameContainer = this.frame.getContentPane();
        frameContainer.setLayout(new BorderLayout());
        frameContainer.add(mainPanel, BorderLayout.CENTER);
    }

    protected void addLogo(){
        BufferedImage image = null;
        try {
            image = ImageIO.read(new File("images/logo.png"));
        } catch (IOException e) {
            e.printStackTrace();
        }

        ImagePanel imagePanel = new ImagePanel(image);
        mainPanel.add(imagePanel, "0,0");
    }

    protected void addText() {
        JPanel textPanel = new JPanel();
        double[][] textLayout = new double[][]{{TableLayoutConstants.FILL},
                {0.25, 0.25, 0.25, 0.25}};
        textPanel.setLayout(new TableLayout(textLayout));
        textPanel.setBackground(Color.WHITE);

        JLabel versionLabel = new JLabel("Version 2.5");
        versionLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
        versionLabel.setHorizontalAlignment(JLabel.CENTER);
        textPanel.add(versionLabel, "0,0");

        JLabel builtLabel = new JLabel("Built on " + getCurrentDate());
        builtLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
        builtLabel.setHorizontalAlignment(JLabel.CENTER);
        textPanel.add(builtLabel, "0,1");

        JLabel jdkLabel = new JLabel("JDK SE 7u2");
        jdkLabel.setFont(new Font("Tahoma", Font.BOLD, 12));
        jdkLabel.setHorizontalAlignment(JLabel.CENTER);
        textPanel.add(jdkLabel, "0,2");

        JLabel copyrightLabel = new JLabel("Copyright \u00A9 2011-2012. All rights reserved.");
        copyrightLabel.setFont(new Font("Tahoma", Font.PLAIN, 12));
        copyrightLabel.setForeground(Color.LIGHT_GRAY);
        copyrightLabel.setHorizontalAlignment(JLabel.CENTER);
        textPanel.add(copyrightLabel, "0,3");

        mainPanel.add(textPanel, "0,1");
    }

    protected void addOKButton() {
        JPanel buttonPanel = new JPanel();
        double[][] buttonLayout = new double[][]{{0.4, 0.2, 0.4},
                {TableLayoutConstants.FILL}};
        buttonPanel.setLayout(new TableLayout(buttonLayout));
        buttonPanel.setBackground(Color.WHITE);

        JButton okButton = new JButton("OK");
        okButton.setSize(100, 30);
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent actionEvent) {
                frame.dispose();
            }
        });
        buttonPanel.add(okButton, "1,0");

        mainPanel.add(buttonPanel, "0,3");
    }

    private String getCurrentDate() {
        DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd");
        Date date = new Date();
        return dateFormat.format(date);
    }

    class ImagePanel extends JPanel {
        BufferedImage bi;

        ImagePanel(BufferedImage image) {
            this.bi = image;
            setBackground(Color.WHITE);
        }

        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.drawImage(bi, 0, 0, this);
        }
    }

}
