package vismon.gui;


import info.clearthought.layout.TableLayout;
import org.noos.xing.mydoggy.*;
import org.noos.xing.mydoggy.event.ContentManagerUIEvent;
import org.noos.xing.mydoggy.plaf.MyDoggyToolWindowManager;
import org.noos.xing.mydoggy.plaf.ui.cmp.ContentDialog;
import vismon.data.Indicator;
import vismon.data.Session;
import vismon.io.FileHandler;
import vismon.plot.ScatterPlot;
import vismon.project.Project;
import vismon.project.ProjectHandler;
import vismon.util.Constants;
import vismon.util.Util;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Vismon {

    private JFrame frame;

    private ToolWindowManager eastToolWindowManager;
    private ToolWindowManager westToolWindowManager;
    private ToolWindowManager southToolWindowManager;

    private JPanel northPanel;
    private JPanel eastPanel;
    private JPanel westPanel;
    private JPanel southPanel;

    private JSplitPane splitHorizontal;
    private JSplitPane splitVertical;

    private JMenu fileMenu;
    private JMenu editMenu;
    private JMenu windowMenu;
    private JMenu indicatorMenu;
    private JMenu helpMenu;

    private JMenuItem openMenuItem;
    private JMenu openRecentDataMenu;
    private JMenuItem openProjectMenuItem;
    private JMenuItem saveProjectMenuItem;
    private JMenuItem saveProjectAsMenuItem;
    private JMenu exportPanelsMenuItem;
    private JMenuItem exportSceMenuItem;
    private List<JCheckBoxMenuItem> indicatorCheckBoxMenuItems;

    private JCheckBoxMenuItem overviewWinMenuItem;
    private JCheckBoxMenuItem cplomWinMenuItem;
    private JCheckBoxMenuItem tradeoffsWinMenuItem;
    private JCheckBoxMenuItem visualRefLineWinMenuItem;

    private VisualRefLineDialog visualRefLineDialog;

    private OverviewPanel overviewPanel;
    private CplomPanel cplomPanel;
    private OptionsPanel optionsPanel;
    private TradeoffsPanel tradeoffsPanel;
    private ScenariosPanel scenariosPanel;
    private IndicatorsPanel optionsIndicatorsPanel;
    private IndicatorsPanel tradeoffsIndicatorsPanel;

    private List<ScatterPlot> scatterPlotList;

    private ContentDialog tradeoffsContentDialog;

    private String saveProjectPath;

    protected void run() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                setUp();
                start();
            }
        });
    }

    protected void setUp() {
        overviewPanel = new OverviewPanel();
        cplomPanel = new CplomPanel();
        optionsPanel = new OptionsPanel();
        tradeoffsPanel = new TradeoffsPanel();
        scenariosPanel = new ScenariosPanel();
        optionsIndicatorsPanel = new IndicatorsPanel();
        tradeoffsIndicatorsPanel = new IndicatorsPanel();

        indicatorCheckBoxMenuItems = new ArrayList<JCheckBoxMenuItem>();

        scatterPlotList = new ArrayList<ScatterPlot>();

        saveProjectPath = null;

        ToolTipManager.sharedInstance().setInitialDelay(10);

        initComponents();
        initSplitPane();
        initWestToolWindowManager();
        initEastToolWindowManager();
        initSouthToolWindowManager();
    }

    protected void start() {
        ToolWindow optionsTool = eastToolWindowManager.getToolWindow("Options");
        optionsTool.setVisible(false);
        optionsTool.setIndex(0);

        ToolWindow optionsindicatorTool = eastToolWindowManager.getToolWindow("Indicators");
        optionsindicatorTool.setVisible(false);
        optionsindicatorTool.setIndex(0);

        ToolWindow scenariosTool = southToolWindowManager.getToolWindow("Scenarios");
        scenariosTool.setVisible(false);
        scenariosTool.setIndex(0);

        ToolWindow tradeoffsindicatorTool = southToolWindowManager.getToolWindow("Indicators");
        tradeoffsindicatorTool.setVisible(false);
        tradeoffsindicatorTool.setIndex(0);

        setSubPanelVisibility();
        frame.setVisible(true);
    }

    protected void initComponents() {
        this.frame = new JFrame("Vismon");
        this.frame.setSize(getToolDimension());
        this.frame.setLocation(0, 0);
        this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.frame.addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent windowEvent) {
                Vismon.getVismonInstance().close();
            }
        });

        addMenus();
        setEnabledMenus(false);

        this.frame.getContentPane().setLayout(new TableLayout(new double[][]{{0, -1, 0}, {0, -1, 0}}));
    }

    private void close() {
        frame.setVisible(false);
        frame.dispose();
        System.exit(0);
    }

    private void addMenus() {
        JMenuBar menuBar = new JMenuBar();

        addFileMenu(menuBar);
        addEditMenu(menuBar);
        addWindowMenu(menuBar);
        addIndicatorMenu(menuBar);
        addHelpMenu(menuBar);

        this.frame.setJMenuBar(menuBar);
    }

    private void setEnabledMenus(boolean enabled) {
        editMenu.setEnabled(enabled);
        windowMenu.setEnabled(enabled);
        indicatorMenu.setEnabled(enabled);

        openMenuItem.setEnabled(!enabled);
        openRecentDataMenu.setEnabled(!enabled);
        openProjectMenuItem.setEnabled(!enabled);

        saveProjectMenuItem.setEnabled(enabled);
        saveProjectAsMenuItem.setEnabled(enabled);
        exportPanelsMenuItem.setEnabled(enabled);
        exportSceMenuItem.setEnabled(enabled);
    }

    private void addFileMenu(JMenuBar menuBar) {
        final ProgressBar progressBar = new ProgressBar();
        fileMenu = new JMenu("File");

        openMenuItem = new JMenuItem("Open Data");
        openMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                final JFileChooser chooser = new JFileChooser(new File("."));
                final int response = chooser.showOpenDialog(null);
                Thread thread = new Thread() {
                    public void run() {
                        if (response == JFileChooser.APPROVE_OPTION) {
                            progressBar.run();
                            File f = chooser.getSelectedFile();
                            updateRecentlyOpenedFile(f.getPath());
                            FileHandler.readDataFile(f);
                            openFilePostActions();
                            progressBar.close();
                        }
                    }
                };
                thread.start();
            }
        });
        openMenuItem.setAccelerator(KeyStroke.getKeyStroke('O', InputEvent.CTRL_DOWN_MASK));
        fileMenu.add(openMenuItem);

        openRecentDataMenu = new JMenu("Open Recent Data");
        addOpenRecentDataSubMenu();
        fileMenu.add(openRecentDataMenu);

        openProjectMenuItem = new JMenuItem("Open Project");
        openProjectMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                final JFileChooser chooser = new JFileChooser(new File(" "));
                final int response = chooser.showOpenDialog(null);
                Thread thread = new Thread() {
                    public void run() {
                        if (response == JFileChooser.APPROVE_OPTION) {
                            progressBar.run();
                            File selectedFile = chooser.getSelectedFile();
                            String fileName = selectedFile.getName();
                            System.out.println(fileName + " is opening.");
                            String fileExtension = fileName.substring(fileName.indexOf(".") + 1, fileName.length());
                            if (!fileExtension.equals("vism"))
                                JOptionPane.showMessageDialog(null, "The only supported format is \"vism\".", "Error", JOptionPane.WARNING_MESSAGE);
                            else
                                try {
                                    Project project = FileHandler.openProject(selectedFile);
                                    ProjectHandler.loadProject(project);
                                    openFilePostActions();
                                } catch (IOException e1) {
                                    e1.printStackTrace();
                                }
                        }
                        progressBar.close();
                    }
                };
                thread.start();
            }
        });
        fileMenu.add(openProjectMenuItem);

        fileMenu.addSeparator();

        saveProjectMenuItem = new JMenuItem("Save Project");
        saveProjectMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (saveProjectPath == null)
                    saveProjectAction("", Session.getDataSetName() + "-VismonProject");
                else
                    try {
                        FileHandler.saveProject(ProjectHandler.createProject(), saveProjectPath);
                    } catch (IOException e1) {
                        e1.printStackTrace();
                    }
            }
        });
        saveProjectMenuItem.setAccelerator(KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK));
        fileMenu.add(saveProjectMenuItem);

        saveProjectAsMenuItem = new JMenuItem("Save Project as");
        saveProjectAsMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveProjectAction("", Session.getDataSetName() + "-VismonProject");
            }
        });
        saveProjectAsMenuItem.setAccelerator(KeyStroke.getKeyStroke('S', InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK));
        fileMenu.add(saveProjectAsMenuItem);

        fileMenu.addSeparator();

        exportPanelsMenuItem = new JMenu("Export Plots to PNG");
        addExportWindowSubMenus();
        fileMenu.add(exportPanelsMenuItem);

        exportSceMenuItem = new JMenuItem("Export Scenarios to CSV");
        exportSceMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    exportScenariosAction();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        });
        fileMenu.add(exportSceMenuItem);

        fileMenu.addSeparator();

        JMenuItem exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                close();
            }
        });
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);
    }

    private void exportScenariosAction() throws IOException {
        JFileChooser chooser = new JFileChooser(new File(""));
        File f = new File("Scenarios-" + Util.getCurrentDateTime() + ".csv");
        chooser.setSelectedFile(f);

        int response = chooser.showSaveDialog(null);
        if (response == JFileChooser.APPROVE_OPTION) {
            File file = chooser.getSelectedFile();
            FileHandler.exportScenarios(file);
        }
    }

    private void addEditMenu(JMenuBar menuBar) {
        editMenu = new JMenu("Edit");

        /*JMenuItem undoMenuItem = new JMenuItem("Undo");
        undoMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("Undo menu item was pressed.");
            }
        });
        undoMenuItem.setAccelerator(KeyStroke.getKeyStroke('Z', InputEvent.CTRL_DOWN_MASK));
        editMenu.add(undoMenuItem);

        editMenu.addSeparator();   */

        JMenuItem deleteAllSceMenuItem = new JMenuItem("Delete all Scenarios");
        deleteAllSceMenuItem.setIcon(new ImageIcon("images/delete.png"));
        deleteAllSceMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                scenariosPanel.removeAllScenarios();
            }
        });
        editMenu.add(deleteAllSceMenuItem);

        editMenu.addSeparator();

        JMenuItem preferencesMenuItem = new JMenuItem("Preferences");
        preferencesMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Preferences preferences = new Preferences();
                preferences.run();
            }
        });
        editMenu.add(preferencesMenuItem);

        menuBar.add(editMenu);
    }

    private void addWindowMenu(JMenuBar menuBar) {
        windowMenu = new JMenu("Window");

        overviewWinMenuItem = new JCheckBoxMenuItem("Constraint");
        overviewWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (overviewWinMenuItem.isSelected() &&
                    westToolWindowManager.getContentManager().getContentCount() == 0)
                    initWestToolWindowManager();
                overviewWinMenuItem.setSelected(overviewWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        overviewWinMenuItem.setSelected(true);
        windowMenu.add(overviewWinMenuItem);

        cplomWinMenuItem = new JCheckBoxMenuItem("Contours");
        cplomWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (cplomWinMenuItem.isSelected() &&
                    eastToolWindowManager.getContentManager().getContentCount() == 0)
                    initEastToolWindowManager();
                cplomWinMenuItem.setSelected(cplomWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        cplomWinMenuItem.setSelected(true);
        windowMenu.add(cplomWinMenuItem);

        tradeoffsWinMenuItem = new JCheckBoxMenuItem("Trade-offs");
        tradeoffsWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (tradeoffsWinMenuItem.isSelected() &&
                    southToolWindowManager.getContentManager().getContentCount() == 0)
                    initSouthToolWindowManager();
                tradeoffsWinMenuItem.setSelected(tradeoffsWinMenuItem.isSelected());
                setSubPanelVisibility();
            }
        });
        tradeoffsWinMenuItem.setSelected(true);
        windowMenu.add(tradeoffsWinMenuItem);

        visualRefLineWinMenuItem = new JCheckBoxMenuItem("Visual Reference Lines");
        visualRefLineWinMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (visualRefLineWinMenuItem.isSelected()) {
                    visualRefLineDialog = new VisualRefLineDialog();
                    visualRefLineDialog.run();
                } else {
                    visualRefLineDialog.close();
                    visualRefLineDialog = null;
                }
            }
        });
        visualRefLineWinMenuItem.setSelected(false);
        windowMenu.add(visualRefLineWinMenuItem);

        menuBar.add(windowMenu);
    }

    private void addIndicatorMenu(JMenuBar menuBar) {
        indicatorMenu = new JMenu("Indicators");
        menuBar.add(indicatorMenu);
    }

    private void addIndicatorsToIndicatorMenu() {
        for (int i = 0; i < Session.getIndicatorNum(); ++i) {
            final Indicator ithIndicator = Session.getIndicator(i);
            final JCheckBoxMenuItem indMenuItem = new JCheckBoxMenuItem(ithIndicator.getName());
            indMenuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    ithIndicator.setVisible(indMenuItem.isSelected());
                    updateIndicatorVisibility();
                }
            });
            indMenuItem.setSelected(true);
            indicatorCheckBoxMenuItems.add(indMenuItem);
            indicatorMenu.add(indMenuItem);
        }
    }

    private void addHelpMenu(JMenuBar menuBar) {
        helpMenu = new JMenu("Help");

        JMenuItem aboutMenuItem = new JMenuItem("About Vismon");
        aboutMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                About about = new About();
                about.run();
            }
        });
        helpMenu.add(aboutMenuItem);

        helpMenu.addSeparator();

        JMenuItem contactMenuItem = new JMenuItem("Contact Support Team");
        contactMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    FileHandler.openURI(new URI("http://www.vismon.org/contact.html"));
                } catch (URISyntaxException e1) {
                    e1.printStackTrace();
                }
            }
        });
        helpMenu.add(contactMenuItem);

        helpMenu.addSeparator();

        JMenuItem onlineDocMenuItem = new JMenuItem("Online Documentation");
        onlineDocMenuItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    FileHandler.openURI(new URI("http://www.vismon.org/documentation.html"));
                } catch (URISyntaxException e1) {
                    e1.printStackTrace();
                }
            }
        });
        helpMenu.add(onlineDocMenuItem);

        menuBar.add(helpMenu);
    }

    public void openFilePostActions() {
        addIndicatorsToIndicatorMenu();
        setEnabledMenus(true);
        setSubPanelVisibility();

        overviewPanel.addMngOptionDualSliders();
        overviewPanel.createIndPanelLayouts();
        overviewPanel.createIndSubPanels();
        overviewPanel.addIndicatorGUIs();
        overviewPanel.repaint();

        cplomPanel.createLabels();
        cplomPanel.createPlotPanels();
        cplomPanel.addGUI();
        cplomPanel.addPopupMenuItems();
        cplomPanel.repaint();

        tradeoffsPanel.addSortByIndComboBox();
        tradeoffsPanel.setUncertaintyComboBoxSelected();
        tradeoffsPanel.createIndicatorPlots();
        tradeoffsPanel.addPlots();

        optionsIndicatorsPanel.addIndicatorCheckBoxes();
        tradeoffsIndicatorsPanel.addIndicatorCheckBoxes();
    }

    private void addOpenRecentDataSubMenu() {
        final ProgressBar progressBar = new ProgressBar();

        try {
            BufferedReader br = new BufferedReader(new FileReader(new File(Constants.RECENTLY_OPENED_FILE_ADDRESS)));
            String filePath = br.readLine();

            while (filePath != null) {
                final String finalFilePath = filePath;

                JMenuItem recentlyFileMenu = new JMenuItem();
                recentlyFileMenu.setText(filePath);
                recentlyFileMenu.setName(filePath);
                recentlyFileMenu.addActionListener(new java.awt.event.ActionListener() {

                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                    Thread thread = new Thread() {
                        public void run() {
                            progressBar.run();
                            FileHandler.readDataFile(new File(finalFilePath));
                            updateRecentlyOpenedFile(finalFilePath);
                            openFilePostActions();
                            System.out.println("hey, you!");
                            System.out.println(progressBar);
                            progressBar.close();
                            System.out.println("hey, you2!");
                        }
                    };
                    thread.start();
                    }
                });

                openRecentDataMenu.add(recentlyFileMenu);
                filePath = br.readLine();
            }
        } catch (IOException ex) {
            Logger.getLogger(Vismon.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void updateRecentlyOpenedFile(String newPath) {
        try {
            List<String> dataFilePathes = new ArrayList<String>();
            dataFilePathes.add(newPath);

            File f = new File(Constants.RECENTLY_OPENED_FILE_ADDRESS);
            BufferedReader br = new BufferedReader(new FileReader(f));
            String filePath = br.readLine();

            while (filePath != null) {
                if (!filePath.equals(newPath))
                    dataFilePathes.add(filePath);
                filePath = br.readLine();
            }

            br.close();

            BufferedWriter bw = new BufferedWriter(new FileWriter(f));
            for (String s : dataFilePathes)
                bw.write(s + "\n");

            bw.close();

        } catch (IOException ex) {
            Logger.getLogger(Vismon.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void saveProjectAction(String path, String fileName) {
        Project project = ProjectHandler.createProject();

        JFileChooser chooser = new JFileChooser(new File(path));
        File f = new File(fileName + ".vism");
        chooser.setSelectedFile(f);

        File file;
        int response = chooser.showSaveDialog(null);
        if (response == JFileChooser.APPROVE_OPTION) {
            file = chooser.getSelectedFile();
            try {
                FileHandler.saveProject(project, file.getPath());
                if (saveProjectPath == null)
                    saveProjectPath = file.getPath();
            } catch (IOException ex) {
                Logger.getLogger(Vismon.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    private void addExportWindowSubMenus() {
        JMenuItem exportCplom = new JMenuItem("Export Contour Plots");
        exportCplom.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exportWindowAction("", "Contour-Plots", cplomPanel);
            }
        });
        exportPanelsMenuItem.add(exportCplom);

        JMenuItem exportTradeOffs = new JMenuItem("Export Trade-offs Plots");
        exportTradeOffs.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                exportWindowAction("", "Trade-offs-plots", tradeoffsPanel);
            }
        });
        exportPanelsMenuItem.add(exportTradeOffs);
    }

    private void exportWindowAction(String path, String fileName, JPanel panel) {
        JFileChooser chooser = new JFileChooser(new File(path));
        File f = new File(fileName + "-" + Util.getCurrentDateTime() + ".png");
        chooser.setSelectedFile(f);

        File file;
        int response = chooser.showSaveDialog(null);
        if (response == JFileChooser.APPROVE_OPTION) {
            file = chooser.getSelectedFile();
            try {
                FileHandler.saveDiagrams(file, panel);
            } catch (IOException ex) {
                Logger.getLogger(Vismon.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    protected void initSplitPane() {
        splitHorizontal = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitHorizontal.setBorder(BorderFactory.createEmptyBorder());
        splitHorizontal.setOneTouchExpandable(false);
        splitHorizontal.setDividerSize(4);
        splitHorizontal.setContinuousLayout(true);
        splitHorizontal.setDividerLocation((int)(this.frame.getHeight() * 0.65));

        northPanel = new JPanel(new BorderLayout());
        splitHorizontal.setTopComponent(northPanel);

        southPanel = new JPanel(new BorderLayout());
        splitHorizontal.setBottomComponent(southPanel);

        Container frameContainer = this.frame.getContentPane();
        frameContainer.setLayout(new BorderLayout());
        frameContainer.add(splitHorizontal, BorderLayout.CENTER);

        splitVertical = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitVertical.setBorder(BorderFactory.createEmptyBorder());
        splitVertical.setOneTouchExpandable(false);
        splitVertical.setDividerSize(4);
        splitVertical.setContinuousLayout(true);
        splitVertical.setDividerLocation((int) (this.frame.getWidth() * 0.35));

        eastPanel = new JPanel(new BorderLayout());
        splitVertical.setRightComponent(eastPanel);

        westPanel = new JPanel(new BorderLayout());
        splitVertical.setLeftComponent(westPanel);
        northPanel.add(splitVertical, BorderLayout.CENTER);
    }

    protected void initWestToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.westToolWindowManager = myDoggyToolWindowManager;

        initWestContentManager();

        westPanel.add(myDoggyToolWindowManager);
    }

    protected void initEastToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.eastToolWindowManager = myDoggyToolWindowManager;

        // Register a Tool.

        eastToolWindowManager.registerToolWindow("Indicators",                      // Id
                                             "Indicators",                 // Title
                                             null,                         // Icon
                optionsIndicatorsPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        eastToolWindowManager.registerToolWindow("Options",                      // Id
                "Options",                 // Title
                null,                         // Icon
                optionsPanel,    // Component
                ToolWindowAnchor.RIGHT);       // Anchor

        setupEastTool();

        for (ToolWindow window : eastToolWindowManager.getToolWindows())
            window.setAvailable(true);

        initEastContentManager();

        eastPanel.add(myDoggyToolWindowManager);
    }

    protected void initSouthToolWindowManager() {
        MyDoggyToolWindowManager myDoggyToolWindowManager = new MyDoggyToolWindowManager();
        this.southToolWindowManager = myDoggyToolWindowManager;

        // Register a Tool.

        southToolWindowManager.registerToolWindow("Indicators",                      // Id
                                             "Indicators",                 // Title
                                             null,                         // Icon
                tradeoffsIndicatorsPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        southToolWindowManager.registerToolWindow("Scenarios",                      // Id
                                             "Scenarios",                 // Title
                                             null,                         // Icon
                                             scenariosPanel,    // Component
                                             ToolWindowAnchor.RIGHT);       // Anchor

        setupSouthTool();

        for (ToolWindow window : southToolWindowManager.getToolWindows())
            window.setAvailable(true);

        initSouthContentManager();

        southPanel.add(myDoggyToolWindowManager);
    }

    protected void setupEastTool() {
        ToolWindow optionsTool = eastToolWindowManager.getToolWindow("Options");
        toolWindowSetup(optionsTool);

        ToolWindow indTool = eastToolWindowManager.getToolWindow("Indicators");
        toolWindowSetup(indTool);

    }

    protected void setupSouthTool() {
        ToolWindow scenariosTool = southToolWindowManager.getToolWindow("Scenarios");
        toolWindowSetup(scenariosTool);

        ToolWindow indTool = southToolWindowManager.getToolWindow("Indicators");
        toolWindowSetup(indTool);
    }

    private void toolWindowSetup(ToolWindow toolWindow) {
        // RepresentativeAnchorDescriptor
        RepresentativeAnchorDescriptor representativeAnchorDescriptor = toolWindow.getRepresentativeAnchorDescriptor();
        representativeAnchorDescriptor.setPreviewEnabled(true);
        representativeAnchorDescriptor.setPreviewDelay(1500);
        representativeAnchorDescriptor.setPreviewTransparentRatio(0.4f);

        // DockedTypeDescriptor
        DockedTypeDescriptor dockedTypeDescriptor = (DockedTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.DOCKED);
        dockedTypeDescriptor.setDockLength(300);
        dockedTypeDescriptor.setPopupMenuEnabled(true);

        // SlidingTypeDescriptor
        SlidingTypeDescriptor slidingTypeDescriptor = (SlidingTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.SLIDING);
        slidingTypeDescriptor.setEnabled(false);

        // FloatingTypeDescriptor
        FloatingTypeDescriptor floatingTypeDescriptor = (FloatingTypeDescriptor) toolWindow.getTypeDescriptor(ToolWindowType.FLOATING);
        floatingTypeDescriptor.setEnabled(false);
    }

    protected void initWestContentManager() {
        ContentManager contentManager = westToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Constraint",
                                                    "Constraint",
                                                    null,      // An icon
                                                    overviewPanel);
        content.setToolTipText("Overview of Management Options and Indicators");

        setupWestContentManagerUI();
    }

    protected void initEastContentManager() {
        ContentManager contentManager = eastToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Contours",
                                                    "Contours",
                                                    null,      // An icon
                                                    cplomPanel);
        content.setToolTipText("Contour Plot Matrix of Indicators");

        setupEastContentManagerUI();
    }

    protected void initSouthContentManager() {
        ContentManager contentManager = southToolWindowManager.getContentManager();
        Content content = contentManager.addContent("Trade-offs",
                                                    "Trade-offs",
                                                    null,      // An icon
                                                    tradeoffsPanel);
        content.setToolTipText("Trade-offs");
        setupSouthContentManagerUI();
    }

    protected void setupWestContentManagerUI() {

        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) westToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                overviewWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                return false;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitVertical.setDividerLocation(0);
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) westToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(false);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = westToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    ContentDialog dialog = (ContentDialog) container;
                    dialog.setLocation(0, 0);
                    dialog.setSize(getToolDimension());
                } else {
                    splitVertical.setDividerLocation((int) (frame.getWidth() * 0.35));
                }
            }
        });
    }

    protected void setupEastContentManagerUI() {

        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) eastToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                cplomWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                return false;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitVertical.setDividerLocation(frame.getWidth());
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) eastToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(false);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = eastToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    ContentDialog dialog = (ContentDialog) container;
                    dialog.setLocation(0, 0);
                    dialog.setSize(getToolDimension());
                } else {
                    splitVertical.setDividerLocation((int) (frame.getWidth() * 0.35));
                }
            }
        });
    }

    protected void setupSouthContentManagerUI() {
        TabbedContentManagerUI contentManagerUI = (TabbedContentManagerUI) southToolWindowManager.getContentManager().getContentManagerUI();

        contentManagerUI.setShowAlwaysTab(true);
        contentManagerUI.setTabPlacement(TabbedContentManagerUI.TabPlacement.TOP);
        contentManagerUI.addContentManagerUIListener(new ContentManagerUIListener() {
            public boolean contentUIRemoving(ContentManagerUIEvent event) {
                //splitHorizontal.setDividerLocation(frame.getHeight());
                tradeoffsWinMenuItem.setSelected(false);
                setSubPanelVisibility();
                return false;
            }

            public void contentUIDetached(ContentManagerUIEvent event) {
                splitHorizontal.setDividerLocation(frame.getHeight());
            }
        });

        TabbedContentUI contentUI = (TabbedContentUI) southToolWindowManager.getContentManager().getContent(0).getContentUI();

        contentUI.setCloseable(false);
        contentUI.setMaximizable(false);
        contentUI.setDetachable(true);
        contentUI.setMinimizable(false);
        contentUI.setTransparentMode(true);
        contentUI.setTransparentRatio(0.7f);
        contentUI.setTransparentDelay(1000);

        final Content content = southToolWindowManager.getContentManager().getContent(0);
        content.addPropertyChangeListener("detached", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getNewValue().equals(Boolean.TRUE)) {
                    Container container = content.getComponent().getParent();
                    while (!(container instanceof ContentDialog))
                        container = container.getParent();
                    //ContentDialog dialog = (ContentDialog) container;
                    tradeoffsContentDialog = (ContentDialog) container;
                    tradeoffsContentDialog.setLocation(0, 0);
                    tradeoffsContentDialog.setSize(getToolDimension());
                } else {
                    splitHorizontal.setDividerLocation((int) (frame.getHeight() * 0.65));
                    tradeoffsContentDialog = null;
                }
                tradeoffsPanel.addPlots();
            }
        });
    }

    private void setSubPanelVisibility() {
        if (openMenuItem.isEnabled()) {         // before opening a file
            northPanel.setVisible(false);
            westPanel.setVisible(false);
            eastPanel.setVisible(false);
            southPanel.setVisible(false);
            return;
        }

        double verSplitPosition;
        double horSplitPosition;

        northPanel.setVisible(true);
        westPanel.setVisible(true);
        eastPanel.setVisible(true);
        southPanel.setVisible(true);

        if (overviewWinMenuItem.isSelected()) {
            if (cplomWinMenuItem.isSelected()) {
                if (tradeoffsWinMenuItem.isSelected()) {            // overview, cplom, trade-offs
                    verSplitPosition = 0.35;
                    horSplitPosition = 0.65;
                } else {                                            // overview, cplom
                    verSplitPosition = 0.35;
                    horSplitPosition = 1.0;

                    southPanel.setVisible(false);
                }
            } else {
                if (tradeoffsWinMenuItem.isSelected()) {            // overview, trade-offs
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.65;

                    eastPanel.setVisible(false);
                } else {                                            // overview
                    verSplitPosition = 1.0;
                    horSplitPosition = 1.0;

                    eastPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            }
        } else {
            if (cplomWinMenuItem.isSelected()) {
                if (tradeoffsWinMenuItem.isSelected()) {            // cplom, trade-offs
                    verSplitPosition = 0.0;
                    horSplitPosition = 0.65;

                    westPanel.setVisible(false);
                } else {                                            // cplom
                    verSplitPosition = 0.0;
                    horSplitPosition = 1.0;

                    westPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            } else {
                if (tradeoffsWinMenuItem.isSelected()) {            // trade-offs
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.0;

                    northPanel.setVisible(false);
                } else {                                            // none
                    verSplitPosition = 1.0;
                    horSplitPosition = 0.0;

                    northPanel.setVisible(false);
                    southPanel.setVisible(false);
                }
            }
        }

        splitVertical.setDividerLocation((int) (frame.getWidth() * verSplitPosition));
        splitHorizontal.setDividerLocation((int) (frame.getHeight() * horSplitPosition));
    }

    public void closeVisualRefLineDialog() {
        visualRefLineDialog.close();
        visualRefLineDialog = null;

        this.visualRefLineWinMenuItem.setSelected(false);
    }

    public void setPreferences() {
        System.out.println("preferences set!");
    }

    private static Vismon vismonInstance;

    public static Vismon getVismonInstance() {
        if (vismonInstance == null) {
            vismonInstance = new Vismon();
        }
        return vismonInstance;
    }

    public void haveScenarios(boolean have) {
        tradeoffsWinMenuItem.setSelected(have);
        setSubPanelVisibility();
    }

    public CplomPanel getCplomPanel() {
        return cplomPanel;
    }

    public OverviewPanel getOverviewPanel() {
        return overviewPanel;
    }

    public ScenariosPanel getScenarioPanel() {
        return scenariosPanel;
    }

    public TradeoffsPanel getTradeoffsPanel() {
        return tradeoffsPanel;
    }

    public IndicatorsPanel getTradeoffsIndicatorsPanel() {
        return tradeoffsIndicatorsPanel;
    }

    public void repaintCplomPanel() {
        cplomPanel.repaint();
        cplomPanel.repaintAllGUIs();
    }

    public void repaintOptionsPanel() {
        optionsPanel.repaint();
    }

    public void repaintOverviewPanel() {
        overviewPanel.repaint();
    }

    public void repaintTradeoffsPanel() {
        if (tradeoffsContentDialog == null) {
            southPanel.repaint();
        } else {
            tradeoffsContentDialog.repaint();
            if (tradeoffsPanel.needsRepaintingInMax()) {
                int width = tradeoffsContentDialog.getWidth();
                int height = tradeoffsContentDialog.getHeight();
                tradeoffsContentDialog.setSize(width-1, height);
                tradeoffsContentDialog.setSize(width, height);
            }
        }
    }

    public void setTradeoffsWinMenuItemSelected() {
        tradeoffsWinMenuItem.setSelected(true);
        setSubPanelVisibility();
    }

    public void updateIndicatorVisibility() {
        for (int rank = 0; rank < Session.getIndicatorNum(); ++rank) {
            boolean visible = Session.getIndicator(rank).isVisible();
            cplomPanel.setContourPlotVisible(rank, visible);
            cplomPanel.setPlotMenuItemVisible(rank, !visible);
            optionsIndicatorsPanel.setIndicatorCBoxSelected(rank, visible);
            optionsIndicatorsPanel.indicatorCBoxSelectedChanged();
            tradeoffsIndicatorsPanel.setIndicatorCBoxSelected(rank, visible);
            tradeoffsIndicatorsPanel.indicatorCBoxSelectedChanged();
            indicatorCheckBoxMenuItems.get(rank).setSelected(visible);
        }
        cplomPanel.repaintPanelLayout();
        updatePlots();
    }

    public void addScatterPlot(ScatterPlot scatterPlot) {
        scatterPlotList.add(scatterPlot);
    }

    public void removeScatterPlot(ScatterPlot scatterPlot) {
        scatterPlotList.remove(scatterPlot);
    }

    public void updatePlots() {
        this.frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        cplomPanel.repaintAllGUIs();
        overviewPanel.repaint();
        scenariosPanel.repaint();
        tradeoffsPanel.addPlots();
        southPanel.repaint();

        for (ScatterPlot sp : scatterPlotList)
            sp.repaint();
        this.frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }

    public static void main(String[] args) {
        UIManager.put("swing.boldMetal", Boolean.FALSE);
        vismonInstance = new Vismon();
        try {
            vismonInstance.run();
            if (args.length > 0) {
                System.out.println("hello! :)");
                System.out.println("Opening file located at " + args[0]);
                long openStart = System.currentTimeMillis();
                File f = new File(args[0]);
                sleep(1000);
                FileHandler.readDataFile(f);
                Vismon.getVismonInstance().openFilePostActions();
                long openEnd = System.currentTimeMillis();
                System.out.println("Took " + (openEnd - openStart) + " ms to open and load");

                System.out.println("Waiting for 5000 ms");
                sleep(5000);

                Robot robot = new Robot();
                int mouseStartX = 520;
                int mouseStartY = 140;
                System.out.println("Moving the cursor from <" + mouseStartX + ", " + mouseStartY + "> to <"
                                    + (mouseStartX + 100) + ", " + (mouseStartY + 50) + ">");
                robot.mouseMove(mouseStartX, mouseStartY);
                for (int i = 0; i <= 50; ++i) {
                    sleep(30);
                    int mouseX = mouseStartX + 2 * i;
                    int mouseY = mouseStartY + 1 * i;
                    robot.mouseMove(mouseX, mouseY);
                }

                System.out.println("Waiting for 5000 ms");
                sleep(5000);

                for (int i = 5; i > 0; --i) {
                    sleep(400);
                    int mouseX = mouseStartX + 20 * i;
                    int mouseY = mouseStartY + 10 * i;
                    robot.mouseMove(mouseX, mouseY);
                    sleep(400);
                    robot.mousePress(InputEvent.BUTTON1_MASK);
                    robot.mouseRelease(InputEvent.BUTTON1_MASK);
                }

                //sleep(15000);

                /*for (int i = 0; i <= 50; ++i) {
                    sleep(30);
                    int mouseX = mouseStartX + 2 * i;
                    int mouseY = mouseStartY + 1 * i;
                    robot.mouseMove(mouseX, mouseY);
                }*/

                //sleep(5000);
                //Vismon.getVismonInstance().close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Dimension getToolDimension() {
        return new Dimension(1440, 900);
        //return Toolkit.getDefaultToolkit().getScreenSize();
    }

    public static void sleep(long ms) {
        try {
            Thread.sleep(ms);
        } catch (InterruptedException e) {}
    }
}
