package vismon.io;

import org.codehaus.jackson.map.ObjectMapper;
import org.msgpack.MessagePack;
import vismon.data.*;
import vismon.gui.OptionsPanel;
import vismon.gui.Vismon;
import vismon.project.Project;
import vismon.util.Constants;
import vismon.util.Util;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URI;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;


public class FileHandler {

    public static String openedDataFilePath;

    public static void readDataFile(File f) {
        openedDataFilePath = f.getPath();

        try {
            BufferedReader input = new BufferedReader(new FileReader(f));
            
            String inputString = input.readLine();
            Session.setDataSetName(inputString);

            inputString = input.readLine();
            int mngOptionNum = Integer.parseInt(inputString);
            Session.setMngOptionNum(mngOptionNum);

            //CREATING MANAGEMENT OPTIONS AND SETTING THEIR PROPERTIES
            for (int i = 0; i < mngOptionNum; ++i) {
                MngOption mngOpt = new MngOption();

                inputString = input.readLine();
                mngOpt.setName(inputString);

                inputString = input.readLine();
                mngOpt.setValueNum(Integer.parseInt(inputString));

                Session.addMngOption(mngOpt);
            }

            inputString = input.readLine();
            int indicatorNum = Integer.parseInt(inputString);
            Session.setIndicatorNum(indicatorNum);

            inputString = input.readLine();
            Session.setIndicatorRowNum(Integer.parseInt(inputString));

            inputString = input.readLine();
            Session.setIndicatorColNum(Integer.parseInt(inputString));

            //CREATING INDICATORS AND SETTING THEIR PROPERTIES
            for (int i = 0; i < indicatorNum; ++i) {
                Indicator ind = new Indicator();

                inputString = input.readLine();
                ind.setName(inputString);

                inputString = input.readLine();
                ind.setShortName(inputString);

                inputString = input.readLine();
                ind.setUnit(inputString);

                String minMaxBest = input.readLine();
                if (minMaxBest.equals("min") || minMaxBest.equals("Min"))
                    ind.setBestValue(Indicator.MIN_BEST);
                else if (minMaxBest.equals("max") || minMaxBest.equals("Max"))
                    ind.setBestValue(Indicator.MAX_BEST);
                else
                    System.out.println("error!");

                Session.addIndicator(ind);
            }

            inputString = input.readLine();
            Session.setSimulationNum(Integer.parseInt(inputString));

            //READING MANAGEMENT OPTION VALUES
            for(MngOption mngOption : Session.getMngOptions()) {
                for (int i = 0; i < mngOption.getValueNum(); ++i) {
                    inputString = input.readLine();
                    mngOption.addValue(Double.parseDouble(inputString));
                }
            }

            //List<double[]> permutations = Util.getPermutations(Session.getMngOptions());
            List<double[]> permutations = Util.getPermutations(Session.getMngOption(0), Session.getMngOption(1));

            //CREATING MANAGEMENT ACTIONS
            for (int i = 0; i < permutations.size(); ++i) {
                MngAction action = new MngAction();
                double[] perm = permutations.get(i);
                for (int j = 0; j < perm.length; ++j)
                    action.addMngValue(Session.getMngOption(j), perm[j]);

                DataItem dataItem = new DataItem(action);
                Session.addDataItem(dataItem);
            }

            //READING INDICATOR VALUES
            for(Indicator depDim : Session.getIndicators()) {
                for (double[] perm : permutations) {
                    MngAction action = new MngAction();
                    for (int j = 0; j < perm.length; ++j)
                        action.addMngValue(Session.getMngOption(j), perm[j]);

                    DataItem dataItem = Session.getDataItem(action);

                    for (int j = 0; j < Session.getSimulationNum(); ++j) {
                        inputString = input.readLine();
                        String str = inputString;

                        if (str.equals("NA"))
                            dataItem.addValue(j, depDim, Constants.NA);
                        else
                            dataItem.addValue(j, depDim, Double.parseDouble(str) + Math.random() * 1e-5);
                    }
                }
            }

            for (MngOption mngOption : Session.getMngOptions())
                mngOption.doComputations();
            for (Indicator indicator : Session.getIndicators())
                indicator.doComputations();
            for (DataItem dataItem : Session.getDataItems())
                dataItem.doComputations();
            for (Indicator indicator : Session.getIndicators())
                Session.calcIndicatorContourStep(indicator);
            Session.setIndicatorDefaultRowColPositions();

            input.close();
        } catch (IOException ex) {
            Logger.getLogger(Session.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public static void saveDiagrams(File file, JPanel panel) throws IOException {
        int w = panel.getWidth();
        int h = panel.getHeight();
        BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = bi.createGraphics();
        panel.paint(g);

        ImageIO.write(bi, "png", file);
    }

    //CHECK File/ExampleScenario.csv FOR AN EXAMPLE
    public static void exportScenarios(File file) throws IOException {
        Writer output = new BufferedWriter(new FileWriter(file));
        writeCSVFile(output);
        output.close();
    }

    private static void writeCSVFile(Writer output) throws IOException {
        //FIRST ROW
        output.write("Scenario name, ");
        for (MngOption mngOption : Session.getMngOptions())
            output.write(mngOption.getName() + ", ");
        for (Indicator indicator : Session.getIndicators()) {
            for (int i = 0; i < 2; ++i)
                output.write(", ");
            output.write(indicator.getName() + ", ");
            for (int i = 0; i < 3; ++i)
                output.write(", ");
        }
        output.write("\n");

        //SECOND ROW
        output.write(", ");
        for (int i = 0; i < Session.getMngOptionNum(); ++i)
            output.write(", ");
        for (int i = 0; i < Session.getIndicatorNum(); ++i)
            output.write("Min, Lower quartile, Median, Average, Upper quartile, Max");
        output.write("\n");

        //SCENARIO ROWS
        for (Scenario scenario : Session.getScenarios()) {
            output.write(scenario.getName() + ", ");
            for (MngOption mngOption : Session.getMngOptions())
                output.write(scenario.getMngAction().getMngValue(mngOption) + ", ");
            for (Indicator indicator : Session.getIndicators()) {
                double[] quartiles = scenario.getQuartiles(indicator);

                output.write(scenario.getMinValue(indicator) + ", ");
                output.write(quartiles[0] + ", ");
                output.write(scenario.getAverageOrMedianValue(indicator, OptionsPanel.MEDIAN_ISOLINES) + ", ");
                output.write(scenario.getAverageOrMedianValue(indicator, OptionsPanel.AVERAGE_ISOLINES) + ", ");
                output.write(quartiles[1] + ", ");
                output.write(scenario.getMaxValue(indicator) + ", ");
            }
            output.write("\n");
        }
    }

    public static void openURI(URI uri) {
        if (Desktop.isDesktopSupported()) {
            try {
                Desktop.getDesktop().browse(uri);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static void saveProject(Project project, String path) throws IOException {
        FileOutputStream fos = new FileOutputStream(path);
        ObjectMapper mapper = new ObjectMapper();
        mapper.writeValue(fos, project);
        fos.close();
    }

    public static Project openProject(File file) throws IOException {
        //FileInputStream fis = new FileInputStream(file);
        /*List<Byte> bytes = new ArrayList<Byte>();
        int ch = fis.read();
        while (ch != -1) {
            bytes.add((byte) ch);
            ch = fis.read();
        }
        System.out.println("--> " + bytes.size());
        byte[] raw = new byte[bytes.size()];
        for (int i = 0; i < raw.length; ++i)
            raw[i] = bytes.get(i);*/

        //Project project = MessagePack.unpack(raw, Project.class);
        ObjectMapper mapper = new ObjectMapper();
        Project project = mapper.readValue(file, Project.class);
        return project;
    }


    public static void main(String[] args) throws IOException {
        Session.setMngOptionNum(2);
        Session.setIndicatorNum(10);
        Project p = new Project();
        p.setIndicatorConstraint(10.0, 5);
        saveProject(p, "/tmp/1");
    }
}