package vismon.project;

import vismon.data.Indicator;
import vismon.data.MngOption;
import vismon.data.Scenario;
import vismon.data.Session;
import vismon.gui.Vismon;
import vismon.io.FileHandler;

import java.awt.*;
import java.io.File;

public class ProjectHandler {

    public static Project createProject() {
        Project project = new Project();
        project.setOpenedDataFilePath(FileHandler.openedDataFilePath);

        for (int i = 0; i < Session.getMngOptionNum(); ++i) {
            MngOption ithMngOption = Session.getMngOption(i);
            project.setMngOptionMinConstraint(ithMngOption.getMinConstraint(), i);
            project.setMngOptionMaxConstraint(ithMngOption.getMaxConstraint(), i);
        }

        for (int i = 0; i < Session.getIndicatorNum(); ++i) {
            Indicator ithIndicator = Session.getIndicator(i);
            project.setIndicatorVisibility(ithIndicator.isVisible(), i);
            project.setIndicatorConstraint(ithIndicator.getConstraintValue(), i);
        }

        for (int i = 0; i < Session.getScenarios().size(); ++i) {
            Scenario ithScenario = Session.getScenario(i);
            project.setScenarioXMngOptionValue(ithScenario.getMngAction().getMngValue(Session.getMngOption(0)), i);
            project.setScenarioYMngOptionValue(ithScenario.getMngAction().getMngValue(Session.getMngOption(1)), i);
            project.setScenarioName(ithScenario.getName(), i);
            project.setScenarioColorRed(ithScenario.getColor().getRed(), i);
            project.setScenarioColorGreen(ithScenario.getColor().getGreen(), i);
            project.setScenarioColorBlue(ithScenario.getColor().getBlue(), i);
            project.setScenarioVisibility(ithScenario.isVisible(), i);
        }

        return project;
    }

    public static void loadProject(Project project) {
        System.out.println("data file @ " + project.getOpenedDataFilePath());
        FileHandler.readDataFile(new File(project.getOpenedDataFilePath()));
        Vismon.getVismonInstance().openFilePostActions();

        for (int i = 0; i < Session.getMngOptionNum(); ++i) {
            MngOption ithMngOption = Session.getMngOption(i);
            ithMngOption.setMinConstraint(project.getMngOptionMinConstraint(i));
            ithMngOption.setMaxConstraint(project.getMngOptionMaxConstraint(i));
        }

        for (int i = 0; i < Session.getIndicatorNum(); ++i) {
            Indicator ithIndicator = Session.getIndicator(i);
            ithIndicator.setConstraintValue(project.getIndicatorConstraint(i));
            ithIndicator.setVisible(project.getIndicatorVisibility(i));
        }

        int scenarioNum = project.getScenarioNum();
        for (int i = 0; i < scenarioNum; ++i) {
            Session.createScenario(project.getScenarioName(i),
                                   project.getScenarioXMngOptionValue(i),
                                   project.getScenarioYMngOptionValue(i),
                                   new Color(project.getScenarioColorRed(i),
                                             project.getScenarioColorGreen(i),
                                             project.getScenarioColorBlue(i)));
        }

        for (int i = 0; i < scenarioNum; ++i)
            Session.getScenario(i).setVisible(project.getScenarioVisibility(i));
    }

}
