package vismon.util;

import vismon.data.Indicator;
import vismon.data.MngAction;
import vismon.data.MngOption;
import vismon.data.Session;
import vismon.gui.OptionsPanel;
import vismon.plot.ContourPixel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Computations {

    public static Map<ContourPixel, Double> computeInterpolatedDataAsCP(MngOption xMng, MngOption yMng, Indicator ind, int avgMedian) {
        int xNum = xMng.getValueNum();
        int yNum = yMng.getValueNum();

        Map<ContourPixel, Double> interpolatedData = new HashMap<ContourPixel, Double>();
        
        for (int j = 0; j < yNum - 1; ++j) {
            for (int i = 0; i < xNum - 1; ++i) {
                //CALCULATING THE VALUES OF MNG OPTIONS @(i,j)
                double xValue = xMng.getValue(i);
                double yValue = yMng.getValue(j);

                //CALCULATING THE VALUE OF IND @(i,j)
                Map<MngOption, Double> map = new HashMap<MngOption, Double>();
                map.put(xMng, xValue);
                map.put(yMng, yValue);
                MngAction action = new MngAction(map);
                double v = Session.getDataItem(action).getAverageValue(ind);
                if (avgMedian == OptionsPanel.MEDIAN_ISOLINES)
                    v = Session.getDataItem(action).getMedianValue(ind);

                ContourPixel cp = new ContourPixel(i, j);
                interpolatedData.put(cp, v);
            }
        }

        //TOP ROW
        for (int i = 0; i < xNum - 1; ++i) {
            //CALCULATING THE VALUES OF MNG OPTIONS @(i,j)
            double xValue = xMng.getValue(i);
            double yValue = yMng.getMax();

            //CALCULATING THE VALUE OF IND @(i, j)
            Map<MngOption, Double> map = new HashMap<MngOption, Double>();
            map.put(xMng, xValue);
            map.put(yMng, yValue);
            MngAction action = new MngAction(map);
            double v = Session.getDataItem(action).getAverageValue(ind);
            if (avgMedian == OptionsPanel.MEDIAN_ISOLINES)
                v = Session.getDataItem(action).getMedianValue(ind);

            ContourPixel cp = new ContourPixel(i, yNum - 1);
            interpolatedData.put(cp, v);
        }

        for (int j = 0; j < yNum - 1; ++j) {
            //CALCULATING THE VALUES OF MNG OPTIONS @(i,j)
            double xValue = xMng.getMax();
            double yValue = yMng.getValue(j);

            //CALCULATING THE VALUE OF IND @(i,j)
            Map<MngOption, Double> map = new HashMap<MngOption, Double>();
            map.put(xMng, xValue);
            map.put(yMng, yValue);
            MngAction action = new MngAction(map);
            double v = Session.getDataItem(action).getAverageValue(ind);
            if (avgMedian == OptionsPanel.MEDIAN_ISOLINES)
                v = Session.getDataItem(action).getMedianValue(ind);

            ContourPixel cp = new ContourPixel(xNum - 1, j);
            interpolatedData.put(cp, v);
        }

        //TOP-RIGHT DATAITEM
        Map<MngOption, Double> map = new HashMap<MngOption, Double>();
        map.put(xMng, xMng.getMax());
        map.put(yMng, yMng.getMax());
        MngAction action = new MngAction(map);
        double v = Session.getDataItem(action).getAverageValue(ind);
        if (avgMedian == OptionsPanel.MEDIAN_ISOLINES)
            v = Session.getDataItem(action).getMedianValue(ind);

        ContourPixel cp = new ContourPixel(xNum - 1, yNum - 1);
        interpolatedData.put(cp, v);

        /*for (Map.Entry<ContourPixel, Double> entry : interpolatedData.entrySet()) {
            System.out.println(entry.getKey() + " - >" + entry.getValue());
        }*/
        return interpolatedData;
    }

    public static void computeIndAvgContourStep(Indicator ind, double min, double max) {
        double contourStep = (max - min) / Constants.contourNum;

        for (int i = 0; i < Constants.contourStepValues.length; ++i) {
            double stepValue = Constants.getContourStepValue(i);
            if (contourStep <= stepValue) {
                contourStep = stepValue;
                break;
            }
        }

        ind.setAvgContourStep(contourStep);
    }

    public static void computeIndAvgContourStepNumber(Indicator ind, double min, double max) {
        double contourStep = ind.getAvgContourStep();
        int contourStepNumber = 1;
        double firstContourValue = contourStep;
        while (firstContourValue < min)
            firstContourValue += contourStep;

        for (double i = firstContourValue; i <= max; i += contourStep)
            ++contourStepNumber;

        if (contourStepNumber > Constants.contourNum) {
            contourStep = Constants.getNextCommonStepValue(contourStep);
            ind.setAvgContourStep(contourStep);
            computeIndAvgContourStepNumber(ind, min, max);
            return;
        }

        ind.setAvgContourStepNumber(contourStepNumber);
    }

    public static void computeIndAvgIsolineValues(Indicator ind, double min, double max) {
        double[] isolineValues = new double[ind.getAvgContourStepNumber()];
        double contourStep = ind.getAvgContourStep();

        double firstContourValue = contourStep;
        while (firstContourValue < min)
            firstContourValue += contourStep;
        isolineValues[0] = firstContourValue;

        for (int i = 1; i < ind.getAvgContourStepNumber(); ++i)
            isolineValues[i] = firstContourValue + (i) * contourStep;

        ind.setAvgIsolineValues(isolineValues);
    }

    public static List<Double> getSimValuesOfPoint(MngOption xMng, MngOption yMng, Indicator ind, double xCoord, double yCoord) {
        if (xCoord == -1 || yCoord == -1)
            return null;

        List<Double> simValues = new ArrayList<Double>();

        double x0 = xMng.getNearestPrevValue(xCoord);
        double x1 = xMng.getNextValue(x0);
        double y0 = yMng.getNearestPrevValue(yCoord);
        double y1 = yMng.getNextValue(y0);

        double a = (xCoord - x0) / (x1 - x0);
        double b = (yCoord - y0) / (y1 - y0);

        double[] values = new double[Session.getSimulationNum()];

        if (!Util.isNaN(x1) && !Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap01 = new HashMap<MngOption, Double>();
            indepMap01.put(xMng, x1);
            indepMap01.put(yMng, y0);
            MngAction ic01 = new MngAction(indepMap01);

            Map<MngOption, Double> indepMap10 = new HashMap<MngOption, Double>();
            indepMap10.put(xMng, x0);
            indepMap10.put(yMng, y1);
            MngAction ic10 = new MngAction(indepMap10);

            Map<MngOption, Double> indepMap11 = new HashMap<MngOption, Double>();
            indepMap11.put(xMng, x1);
            indepMap11.put(yMng, y1);
            MngAction ic11 = new MngAction(indepMap11);

            for (int m = 0; m < values.length; ++m) {
                double p00 = Session.getDataItem(ic00).getValue(m, ind);
                double p01 = Session.getDataItem(ic01).getValue(m, ind);
                double p10 = Session.getDataItem(ic10).getValue(m, ind);
                double p11 = Session.getDataItem(ic11).getValue(m, ind);

                values[m] = (1 - a) * ((1 - b) * p00 + b * p10) + a * ((1 - b) * p01 + b * p11);
            }

        } else if (!Util.isNaN(x1) && Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap01 = new HashMap<MngOption, Double>();
            indepMap01.put(xMng, x1);
            indepMap01.put(yMng, y0);
            MngAction ic01 = new MngAction(indepMap01);

            for (int m = 0; m < values.length; ++m) {
                double p00 = Session.getDataItem(ic00).getValue(m, ind);
                double p01 = Session.getDataItem(ic01).getValue(m, ind);

                values[m] = (1 - a) * p00 + a * p01;
            }

        } else if (Util.isNaN(x1) && !Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap10 = new HashMap<MngOption, Double>();
            indepMap10.put(xMng, x0);
            indepMap10.put(yMng, y1);
            MngAction ic10 = new MngAction(indepMap10);

            for (int m = 0; m < values.length; ++m) {
                double p00 = Session.getDataItem(ic00).getValue(m, ind);
                double p10 = Session.getDataItem(ic10).getValue(m, ind);

                values[m] = (1 - b) * p00 + b * p10;
            }

        } else {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            for (int m = 0; m < values.length; ++m) {
                double p00 = Session.getDataItem(ic00).getValue(m, ind);

                values[m] = p00;
            }
        }

        for (int m = 0; m < values.length; ++m)
            simValues.add(values[m]);
        return simValues;
    }
    
    public static double getInterplotadeValueOfPoint(MngOption xMng, MngOption yMng, Indicator ind, double xGrid, double yGrid, int avgMedian) {
        int xPrevGrid = (int)(Math.floor(xGrid));
        int xNextGrid = xPrevGrid + 1;
        int yPrevGrid = (int)(Math.floor(yGrid));
        int yNextGrid = yPrevGrid + 1;

        double x0 = xMng.getValue(xPrevGrid);
        double x1 = xMng.getValue(xNextGrid);
        double y0 = yMng.getValue(yPrevGrid);
        double y1 = yMng.getValue(yNextGrid);

        double a = xGrid - xPrevGrid;
        double b = yGrid - yPrevGrid;

        double value = 0.0;

        if (!Util.isNaN(x1) && !Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap01 = new HashMap<MngOption, Double>();
            indepMap01.put(xMng, x1);
            indepMap01.put(yMng, y0);
            MngAction ic01 = new MngAction(indepMap01);

            Map<MngOption, Double> indepMap10 = new HashMap<MngOption, Double>();
            indepMap10.put(xMng, x0);
            indepMap10.put(yMng, y1);
            MngAction ic10 = new MngAction(indepMap10);

            Map<MngOption, Double> indepMap11 = new HashMap<MngOption, Double>();
            indepMap11.put(xMng, x1);
            indepMap11.put(yMng, y1);
            MngAction ic11 = new MngAction(indepMap11);

            if (avgMedian == OptionsPanel.AVERAGE_ISOLINES) {
                double p00 = Session.getDataItem(ic00).getAverageValue(ind);
                double p01 = Session.getDataItem(ic01).getAverageValue(ind);
                double p10 = Session.getDataItem(ic10).getAverageValue(ind);
                double p11 = Session.getDataItem(ic11).getAverageValue(ind);

                value = (1 - a) * ((1 - b) * p00 + b * p10) + a * ((1 - b) * p01 + b * p11);
            } else {
                double p00 = Session.getDataItem(ic00).getMedianValue(ind);
                double p01 = Session.getDataItem(ic01).getMedianValue(ind);
                double p10 = Session.getDataItem(ic10).getMedianValue(ind);
                double p11 = Session.getDataItem(ic11).getMedianValue(ind);

                value = (1 - a) * ((1 - b) * p00 + b * p10) + a * ((1 - b) * p01 + b * p11);
            }

        } else if (!Util.isNaN(x1) && Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap01 = new HashMap<MngOption, Double>();
            indepMap01.put(xMng, x1);
            indepMap01.put(yMng, y0);
            MngAction ic01 = new MngAction(indepMap01);

            if (avgMedian == OptionsPanel.AVERAGE_ISOLINES) {
                double p00 = Session.getDataItem(ic00).getAverageValue(ind);
                double p01 = Session.getDataItem(ic01).getAverageValue(ind);

                value += (1 - a) * p00 + a * p01;
            } else {
                double p00 = Session.getDataItem(ic00).getMedianValue(ind);
                double p01 = Session.getDataItem(ic01).getMedianValue(ind);

                value += (1 - a) * p00 + a * p01;
            }

        } else if (Util.isNaN(x1) && !Util.isNaN(y1)) {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            Map<MngOption, Double> indepMap10 = new HashMap<MngOption, Double>();
            indepMap10.put(xMng, x0);
            indepMap10.put(yMng, y1);
            MngAction ic10 = new MngAction(indepMap10);

            if (avgMedian == OptionsPanel.AVERAGE_ISOLINES) {
                double p00 = Session.getDataItem(ic00).getAverageValue(ind);
                double p10 = Session.getDataItem(ic10).getAverageValue(ind);

                value += (1 - b) * p00 + b * p10;
            } else {
                double p00 = Session.getDataItem(ic00).getMedianValue(ind);
                double p10 = Session.getDataItem(ic10).getMedianValue(ind);

                value += (1 - b) * p00 + b * p10;
            }

        } else {
            Map<MngOption, Double> indepMap00 = new HashMap<MngOption, Double>();
            indepMap00.put(xMng, x0);
            indepMap00.put(yMng, y0);
            MngAction ic00 = new MngAction(indepMap00);

            if (avgMedian == OptionsPanel.AVERAGE_ISOLINES) {
                value = Session.getDataItem(ic00).getAverageValue(ind);
            } else {
                value = Session.getDataItem(ic00).getMedianValue(ind);
            }
        }

        return value;
    }
}
